#!/usr/bin/env bash

# Bail immediately on on error
set -e

# Make sure the latest C code is compiled
npm run build

# Make sure tests pass before publishing
npm test

# Build documentation and commit to the gh-pages branch, then go back to master branch
docco lib/rainbow-dash.js
git stash
git checkout gh-pages
rm -rf docs/public
mv docs/* .
rm -rf docs
rm -rf index.html
mv rainbow-dash.html index.html
git commit -am "Automatic documentation for version $npm_package_version"
git checkout master
git stash pop

# Build minified version usable in the browser
browserify ./lib/browserify.js | uglifyjs > ./lib/rainbow-dash.min.js

# Create the commit, tag the commit with the proper version, and push to GitHub
git commit -am "Automatic commit of version $npm_package_version"
git tag $npm_package_version
git push
git push --tags

# Publish to NPM
npm publish
