// dash-core is the C-based processing core of the second generation of
// rainbow-dash. Built with LLVM + emscripten into asm.js
// Because there is no easy way to get gyp to set up and run this build
// system (especially because getting LLVM installed on Win/Mac/Linux
// varies so much) the build script is a simple shell script and its
// results are kept in an "out" directory and used from there.

typedef unsigned char byte;

// # interpolate
// Simple integer out `y=mx+b` method. `y` must be in the range of 0-255.
static inline byte interpolate(double m, double b, double x) {
    return (byte)(m*x+b);
}

// # calculateM
// Given two points on a line, calculate `m` for `y=mx+b`
// x1 *must not* equal x2
double calculateM(double x1, double x2, double y1, double y2) {
    return (y2 - y1)/(x2 - x1);
}

// # calculateB
// Given two points on a line, calculate `b` for `y=mx+b`
// x1 *must not* equal x2
double calculateB(double x1, double x2, double y1, double y2) {
    return (x2*y1 - x1*y2)/(x2 - x1);
}

// # getTriple
// Given the array of the stop values, the arrays of m and b pairs
// for each color channel (r, g, b) and the input value, calculate
// the output (r, g, b) value
byte* getTriple(double* stopVals, double* rVals, double* gVals, double* bVals, double x) {
    int i = 0;
    static byte out[3] = {0};
    while(stopVals[i+1] < x) { i++; }
    out[0] = interpolate(rVals[2*i], rVals[2*i+1], x);
    out[1] = interpolate(gVals[2*i], gVals[2*i+1], x);
    out[2] = interpolate(bVals[2*i], bVals[2*i+1], x);
    return out;
}
