var logger, rainbowDash;

exports.injectObjs = function(objs) {
    logger = objs.logger;
    rainbowDash = objs.rainbowDash;
};

function bootstrap(test) {
    test.expect = test.expect || test.plan;
    test.done = test.done || test.end;
}

exports.grades = function(test) {
    bootstrap(test);
    test.expect(5);
    var gradeScale = rainbowDash({
        100: 'rgb(0,0,255)',
        90:  'rgb(0,255,0)',
        80:  'rgb(255,255,0)',
        60:  'rgb(255,0,0)'
    });
    test.equal('rgb(0,0,255)', gradeScale(100));
    test.equal('rgb(0,255,0)', gradeScale(90));
    test.equal('rgb(255,255,0)', gradeScale(80));
    test.equal('rgb(255,0,0)', gradeScale(60));
    test.equal('rgb(0,127,127)', gradeScale(95));
    test.done();
};

exports.perf = function(test) {
    bootstrap(test);
    var ops = 1000000;
    var range = {};
    range[0] = 'rgb(0,0,0)';
    range[ops] = 'rgb(255,255,255)';
    var gradient = rainbowDash(range);
    var startTime = Date.now();
    for(var i = 0; i < ops; i++) {
        gradient(i);
    }
    var endTime = Date.now();
    logger((endTime - startTime) + 'ms, ' + (ops / (endTime - startTime) * 1000) + ' ops/sec');
    test.done();
};

/* jshint maxparams: 5 */
function naiveInterpolation(x1, y1, x2, y2, x) {
    if(x <= x1) return y1;
    if(x >= x2) return y2;
    var rgb1 = y1.match(/[0-9]+/g).map(Number);
    var rgb2 = y2.match(/[0-9]+/g).map(Number);
    var rgb = rgb1.map(function(rgb1Val, i) {
        return ((rgb2[i] - rgb1[i])/(x2 - x1))*x + ((x2*rgb1[i] - x1*rgb2[i])/(x2 - x1));
    });
    return 'rgb(' + rgb[0] + ',' + rgb[1] + ',' + rgb[2] + ')';
}

exports.naivePerf = function(test) {
    bootstrap(test);
    var ops = 100000;
    var startTime = Date.now();
    for(var i = 0; i < ops; i++) {
        naiveInterpolation(0, 'rgb(0,0,0)', ops, 'rgb(255,255,255)', i);
    }
    var endTime = Date.now();
    logger((endTime - startTime) + 'ms, ' + (ops / (endTime - startTime) * 1000) + ' ops/sec');
    test.done();
};

exports.parseHex = function(test) {
    bootstrap(test);
    test.expect(2);
    var hexGradient = rainbowDash({ outputFormat: 'hex' }, {
        0: '#ff0000',
        1: '#00ffff'
    });
    test.equal(hexGradient(0), '#ff0000');
    test.equal(hexGradient(1), '#00ffff');
    test.done();
};

exports.mixedInputUniformOutput = function(test) {
    bootstrap(test);
    test.expect(4);
    var rgbGradient = rainbowDash({
        0: '#ff0000',
        1: 'rgb(255,255,0)'
    });
    var hexGradient = rainbowDash({ outputFormat: 'hex' }, {
        0: '#ff0000',
        1: 'rgb(255,255,0)'
    });
    test.equal(rgbGradient(0), 'rgb(255,0,0)');
    test.equal(rgbGradient(1), 'rgb(255,255,0)');
    test.equal(hexGradient(0), '#ff0000');
    test.equal(hexGradient(1), '#ffff00');
    test.done();
};

exports.uniformSpacing = function(test) {
    bootstrap(test);
    test.expect(3);
    var uniformGradient = rainbowDash({
        uniformStops: true,
        inputRange: [0, 1]
    }, ['#ff0000', 'rgb(0,255,255)', '#00ff00']);
    test.equal(uniformGradient(0), 'rgb(255,0,0)');
    test.equal(uniformGradient(0.5), 'rgb(0,255,255)');
    test.equal(uniformGradient(1), 'rgb(0,255,0)');
    test.done();
};

exports.throwsOnGarbageInput = function(test) {
    bootstrap(test);
    test.expect(1);
    test.throws(rainbowDash.bind(this, { 0: 'foo', 1: 'bar' }), /Unknown CSS color string/);
    test.done();
};

exports.dropsInvalidInputRanges = function(test) {
    bootstrap(test);
    test.expect(2);
    var nonArrayRange = rainbowDash({
        inputRange: 'foobar',
        outputFormat: 'hex'
    }, {
        0: '#ff0000',
        1: '#00ffff'
    });
    test.equal(nonArrayRange(0.5), '#7f7f7f');
    var arrayRangeTooLarge = rainbowDash({
        inputRange: [0, 1, 2]
    }, {
        0: '#ff0000',
        1: '#00ffff'
    });
    test.equal(arrayRangeTooLarge(2), 'rgb(0,255,255)');
    test.done();
};
